/////////////////////////////////////////////////////////////////////////////
// msgTimer.cpp
//
// sends messages at timed intervals through worker thread
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "msgTimer.h"


/////////////////////////////////////////////////////////////////////////////
// parameter object to pass stuff into the thread
/////////////////////////////////////////////////////////////////////////////

class params {
	// vars
	DWORD	m_dwSendToThreadID;
	long	m_lSleepTimeSeconds;
	long	m_lSleepTimeResolutionMilliSeconds;
	UINT	m_uintSendMessage;
	bool*	m_pbStopThreadFlag;

public:
	// construction
	params(	DWORD dwSendToThreadID,
			long lSleepTimeSeconds,
			long lSleepTimeResolutionMilliSeconds,
			UINT uintSendMessage,
			bool* pbStopThreadFlag) :
		m_dwSendToThreadID(dwSendToThreadID),
		m_lSleepTimeSeconds(lSleepTimeSeconds),
		m_lSleepTimeResolutionMilliSeconds(lSleepTimeResolutionMilliSeconds),
		m_uintSendMessage(uintSendMessage),
		m_pbStopThreadFlag(pbStopThreadFlag)
	{};

	// accessors
		DWORD getSendToThreadID() const { return m_dwSendToThreadID; }
		long  getSleepTimeSeconds() const { return m_lSleepTimeSeconds; }
		long  getSleepTimeResolutionMilliSeconds() const { return m_lSleepTimeResolutionMilliSeconds; }
		UINT  getSendMessage() const { return m_uintSendMessage; }
		bool* getStopThreadFlagPtr() const { return m_pbStopThreadFlag; }
};

/////////////////////////////////////////////////////////////////////////////
// the worker thread function, started by AfxBeginThread()
/////////////////////////////////////////////////////////////////////////////

static UINT TimedMessage(LPVOID lParam)
{
	// pick up the params and delete param structure (else
	// it would go lost if thread terminates)
	params* pP = reinterpret_cast<params*>(lParam);
	DWORD dwSendToThreadID = pP->getSendToThreadID();
	UINT uintSendMessage = pP->getSendMessage();
	long lSleepTimeSeconds = pP->getSleepTimeSeconds();
	long lSleepTimeResolutionMilliSeconds = pP->getSleepTimeResolutionMilliSeconds();
	bool* pbStopThreadFlag = pP->getStopThreadFlagPtr();
	delete pP;

	long lSleepLoops = lSleepTimeSeconds * 1000 / lSleepTimeResolutionMilliSeconds;

	Sleep(lSleepTimeResolutionMilliSeconds);

	// this thread can only be terminated from the outside
	while (true)
	{
		::PostThreadMessage(dwSendToThreadID, uintSendMessage, 0, 0L);
		TRACE("Sending timed message\n");
		for (long lLoop = 0; lLoop < lSleepLoops; lLoop++)
		{
			// check for termination signal here and break if signalled
			// (it's not necessary to do locking or mutexing here, since any
			// change in the bool is enough to exit)
			if (*pbStopThreadFlag)
			{
				TRACE("--- TimedMessage worker thread exit\n");
				return 0;
			}
			Sleep(lSleepTimeResolutionMilliSeconds);
		}
	}
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// the only externally accessible function to do it all
/////////////////////////////////////////////////////////////////////////////

CWinThread* MsgTimer(DWORD dwSendToThreadID, 
					 long lSleepTimeSeconds, 
					 long lSleepTimeResolutionMilliSeconds,
					 UINT uintSendMessage, 
					 bool* pbStopThreadFlag)
{
	// note: pPar gets deleted by the thread function itself
	params* pPar = new params(dwSendToThreadID, lSleepTimeSeconds, lSleepTimeResolutionMilliSeconds,
		uintSendMessage, pbStopThreadFlag);
	return AfxBeginThread(TimedMessage, (LPVOID)pPar);
}
